@@ -5,7 +5,6 @@
 import csv
 import itertools as itt
 import json
-import warnings
 from collections import defaultdict
 from dataclasses import dataclass, field
 from pathlib import Path
@@ -13,6 +12,7 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
+    Collection,
     DefaultDict,
     Dict,
     Iterable,
@@ -36,6 +36,7 @@ if TYPE_CHECKING:  # pragma: no cover
 __all__ = [
     "Converter",
     "Record",
+    "DuplicateValueError",
     "DuplicatePrefixes",
     "DuplicateURIPrefixes",
     "chain",
@@ -122,7 +123,7 @@ def _get_duplicate_prefixes(records: List[Record]) -> List[Tuple[Record, Record,
     ]
 
 
-def _get_prefix_map(records: List[Record]) -> Mapping[str, str]:
+def _get_prefix_map(records: List[Record]) -> Dict[str, str]:
     rv = {}
     for record in records:
         rv[record.prefix] = record.uri_prefix
@@ -131,7 +132,7 @@ def _get_prefix_map(records: List[Record]) -> Mapping[str, str]:
     return rv
 
 
-def _get_reverse_prefix_map(records: List[Record]) -> Mapping[str, str]:
+def _get_reverse_prefix_map(records: List[Record]) -> Dict[str, str]:
     rv = {}
     for record in records:
         rv[record.uri_prefix] = record.prefix
@@ -182,9 +183,9 @@ class Converter:
     """
 
     #: The expansion dictionary with prefixes as keys and priority URI prefixes as values
-    prefix_map: Mapping[str, str]
+    prefix_map: Dict[str, str]
     #: The mapping from URI prefixes to prefixes
-    reverse_prefix_map: Mapping[str, str]
+    reverse_prefix_map: Dict[str, str]
     #: A prefix trie for efficient parsing of URIs
     trie: StringTrie
 
@@ -192,7 +193,7 @@ class Converter:
         """Instantiate a converter.
 
         :param records:
-            A list of records
+            A list of records. If you plan to build a converter incrementally, pass an empty list.
         :param strict:
             If true, raises issues on duplicate URI prefixes
         :param delimiter:
@@ -214,30 +215,77 @@ class Converter:
         self.reverse_prefix_map = _get_reverse_prefix_map(records)
         self.trie = StringTrie(self.reverse_prefix_map)
 
-    @classmethod
-    def from_extended_prefix_map_url(cls, url: str, **kwargs: Any) -> "Converter":
-        """Get a converter from a remote JSON file containing an extended prefix map.
+    def _check_record(self, record: Record) -> None:
+        """Check if the record can be added."""
+        if record.prefix in self.prefix_map:
+            raise ValueError(f"new record has duplicate prefix: {record.prefix}")
+        if record.uri_prefix in self.reverse_prefix_map:
+            raise ValueError(f"new record has duplicate URI prefix: {record.uri_prefix}")
+        for prefix_synonym in record.prefix_synonyms:
+            if prefix_synonym in self.prefix_map:
+                raise ValueError(f"new record has duplicate prefix: {prefix_synonym}")
+        for uri_prefix_synonym in record.uri_prefix_synonyms:
+            if uri_prefix_synonym in self.reverse_prefix_map:
+                raise ValueError(f"new record has duplicate URI prefix: {uri_prefix_synonym}")
 
-        :param url: The URL of a JSON file containiing dictionaries corresponding to the :class:`Record` schema
-        :param kwargs: Keyword arguments to pass to the parent class's init
-        :returns: A converter
+    def add_record(self, record: Record) -> None:
+        """Append a record to the converter."""
+        self._check_record(record)
 
-        An extended prefix map is a list of dictionaries containing four keys:
+        self.prefix_map[record.prefix] = record.uri_prefix
+        for prefix_synonym in record.prefix_synonyms:
+            self.prefix_map[prefix_synonym] = record.uri_prefix
 
-        1. A ``prefix`` string
-        2. A ``uri_prefix`` string
-        3. An optional list of strings ``prefix_synonyms``
-        4. An optional list of strings ``uri_prefix_synonyms``
+        self.reverse_prefix_map[record.uri_prefix] = record.prefix
+        self.trie[record.uri_prefix] = record.prefix
+        for uri_prefix_synonym in record.uri_prefix_synonyms:
+            self.reverse_prefix_map[uri_prefix_synonym] = record.prefix
+            self.trie[uri_prefix_synonym] = record.prefix
 
-        Across the whole list of dictionaries, there should be uniqueness within
-        the union of all ``prefix`` and ``prefix_synonyms`` as well as uniqueness
-        within the union of all ``uri_prefix`` and ``uri_prefix_synonyms``.
+    def add_prefix(
+        self,
+        prefix: str,
+        uri_prefix: str,
+        prefix_synonyms: Optional[Collection[str]] = None,
+        uri_prefix_synonyms: Optional[Collection[str]] = None,
+    ) -> None:
+        """Append a prefix to the converter.
 
-        >>> url = "https://github.com/biopragmatics/bioregistry/raw/main/exports/contexts/bioregistry.epm.json"
-        >>> converter = Converter.from_extended_prefix_map_url(url)
+        :param prefix:
+            The prefix to append, e.g., ``go``
+        :param uri_prefix:
+            The URI prefix to append, e.g., ``http://purl.obolibrary.org/obo/GO_``
+        :param prefix_synonyms:
+            An optional collection of synonyms for the prefix such as ``gomf``, ``gocc``, etc.
+        :param uri_prefix_synonyms:
+            An optional collections of synonyms for the URI prefix such as
+            ``https://bioregistry.io/go:``, ``http://www.informatics.jax.org/searches/GO.cgi?id=GO:``, etc.
+
+        This can be used to add missing namespaces on-the-fly to an existing converter:
+
+        >>> import curies
+        >>> converter = curies.get_obo_converter()
+        >>> converter.add_prefix("hgnc", "https://bioregistry.io/hgnc:")
+        >>> converter.expand("hgnc:1234")
+        'https://bioregistry.io/hgnc:1234'
+        >>> converter.expand("GO:0032571 ")
+        'http://purl.obolibrary.org/obo/GO_0032571'
+
+        This can also be used to incrementally build up a converter from scratch:
+
+        >>> import curies
+        >>> converter = curies.Converter(records=[])
+        >>> converter.add_prefix("hgnc", "https://bioregistry.io/hgnc:")
+        >>> converter.expand("hgnc:1234")
+        'https://bioregistry.io/hgnc:1234'
         """
-        warnings.warn("directly use Converter.from_extended_prefix_map", DeprecationWarning)
-        return cls.from_extended_prefix_map(url, **kwargs)
+        record = Record(
+            prefix=prefix,
+            uri_prefix=uri_prefix,
+            prefix_synonyms=sorted(prefix_synonyms or []),
+            uri_prefix_synonyms=sorted(uri_prefix_synonyms or []),
+        )
+        self.add_record(record)
 
     @classmethod
     def from_extended_prefix_map(
@@ -286,6 +334,11 @@ class Converter:
         # URI prefix synoynm
         >>> converter.compress("https://www.ebi.ac.uk/chebi/searchId.do?chebiId=CHEBI:138488")
         'CHEBI:138488'
+
+        Example from a remote source:
+
+        >>> url = "https://github.com/biopragmatics/bioregistry/raw/main/exports/contexts/bioregistry.epm.json"
+        >>> converter = Converter.from_extended_prefix_map(url)
         """
         return cls(
             records=[
@@ -405,6 +458,12 @@ class Converter:
         'CHEBI:138488'
         >>> converter.compress("https://www.ebi.ac.uk/chebi/searchId.do?chebiId=138488")
         'CHEBI:138488'
+
+        Altenatively, get content from the internet like
+
+        >>> url = "https://github.com/biopragmatics/bioregistry/raw/main/exports/contexts/bioregistry.rpm.json"
+        >>> converter = Converter.from_reverse_prefix_map(url)
+        >>> "chebi" in Converter.prefix_map
         """
         dd = defaultdict(list)
         for uri_prefix, prefix in _prepare(reverse_prefix_map).items():
@@ -419,23 +478,6 @@ class Converter:
             )
         return cls(records)
 
-    @classmethod
-    def from_reverse_prefix_map_url(cls, url: str) -> "Converter":
-        """Get a remote reverse prefix map JSON file then parse with :meth:`Converter.from_reverse_prefix_map`.
-
-        :param url:
-            A URL to a reverse prefix map JSON file
-        :return:
-            A converter
-
-        >>> url = "https://github.com/biopragmatics/bioregistry/raw/main/exports/contexts/bioregistry.rpm.json"
-        >>> converter = Converter.from_reverse_prefix_map_url(url)
-        >>> "chebi" in Converter.prefix_map
-        True
-        """
-        warnings.warn("directly use Converter.from_reverse_prefix_map", DeprecationWarning)
-        return cls.from_reverse_prefix_map(url)
-
     @classmethod
     def from_jsonld(cls, data: LocationOr[Dict[str, Any]]) -> "Converter":
         """Get a converter from a JSON-LD object, which contains a prefix map in its ``@context`` key.
@@ -444,32 +486,21 @@ class Converter:
             A JSON-LD object
         :return:
             A converter
-        """
-        return cls.from_prefix_map(_prepare(data)["@context"])
 
-    @classmethod
-    def from_jsonld_url(cls, url: str) -> "Converter":
-        """Get a remote JSON-LD file then parse with :meth:`Converter.from_jsonld`.
-
-        :param url:
-            A URL to a JSON-LD file
-        :return:
-            A converter
+        Example from a remote context file:
 
         >>> base = "https://raw.githubusercontent.com"
         >>> url = f"{base}/biopragmatics/bioregistry/main/exports/contexts/semweb.context.jsonld"
-        >>> converter = Converter.from_jsonld_url(url)
+        >>> converter = Converter.from_jsonld(url)
         >>> "rdf" in converter.prefix_map
-        True
         """
-        warnings.warn("directly use Converter.from_jsonld", DeprecationWarning)
-        return cls.from_jsonld(url)
+        return cls.from_prefix_map(_prepare(data)["@context"])
 
     @classmethod
     def from_jsonld_github(
         cls, owner: str, repo: str, *path: str, branch: str = "main"
     ) -> "Converter":
-        """Construct a remote JSON-LD URL on GitHub then parse with :meth:`Converter.from_jsonld_url`.
+        """Construct a remote JSON-LD URL on GitHub then parse with :meth:`Converter.from_jsonld`.
 
         :param owner: A github repository owner or organization (e.g., ``biopragmatics``)
         :param repo: The name of the repository (e.g., ``bioregistry``)
